@@ -1,7 +1,6 @@
 import gc
 import logging
 import math
-import os
 import uuid
 from datetime import datetime
 
@@ -21,7 +20,7 @@ TEMPLATE_COLUMN_MAPPINGS = {
     "text_generation": {
         "doc_url": "https://huggingface.co/docs/trl/main/en/dataset_formats#prompt-completion",
         "required_columns": {
-            "instruction": ["prompt","instruction", "query", "question"],
+            "instruction": ["prompt", "instruction", "query", "question"],
             "response": [
                 "completion",
                 "response",
@@ -97,8 +96,8 @@ class HuggingFaceHandler:
             return
 
         if dataset_tag not in TEMPLATE_COLUMN_MAPPINGS:
-            logger.warning(
-                f"Unknown dataset_tag '{dataset_tag}'; skipping schema check."
+            logger.error(
+                f"Unknown dataset_tag '{dataset_tag}'; skipping schema check.",
             )
             return
 
@@ -131,7 +130,6 @@ class HuggingFaceHandler:
                         if name in features:
                             renambale_columns.append((name, role))
                             break
-            logger.info(f"Dataset schema validation passed for tag '{dataset_tag}'.")
 
         return renambale_columns
 
@@ -151,14 +149,6 @@ class HuggingFaceHandler:
         label_feature = dataset["train"].features.get("label")
         has_class_labels = isinstance(label_feature, ClassLabel)
 
-        if has_class_labels:
-            logger.info(f"Found class labels: {label_feature.names}")
-        else:
-            logger.warning(
-                "Label column is not of type ClassLabel. "
-                "Will treat labels as raw strings."
-            )
-
         def save_image_batch(batch):
             """
             Processes a batch of examples to save images and generate metadata.
@@ -199,15 +189,15 @@ class HuggingFaceHandler:
                     image.convert("RGB").save(destination_path)
                     new_file_paths.append(str(relative_path))
                     new_labels.append(label_name)
-                except Exception as e:
-                    logger.error(f"Failed to save an image: {e}")
+                except Exception:
                     new_file_paths.append(None)
                     new_labels.append(None)
+                    raise
 
             # Return a dictionary with the new columns
             return {"file_path": new_file_paths, "label_name": new_labels}
 
-        num_cores = os.cpu_count() or 4
+        num_cores = 2
         logger.info(f"Processing dataset in parallel using {num_cores} cores.")
 
         # Apply the map function to generate new columns
@@ -259,6 +249,8 @@ class HuggingFaceHandler:
         dataset_config: str,
         user_name: str,
         private: bool,
+        base_extra: dict = {},
+        restructure: bool = True,
         s3_config: dict = None,
         clearml_config: dict = None,
         revision: str = "main",
@@ -280,30 +272,26 @@ class HuggingFaceHandler:
                 DatasetConditionChecker().check_huggingface_size(
                     dataset_name, revision=revision
                 )
-            except ValueError as size_error:
-                logger.error(
-                    f"Dataset size check failed for '{dataset_name}': {size_error}"
+            except Exception as e:
+                check_size_error = base_extra.copy()
+                check_size_error.update(
+                    {
+                        "event": {
+                            "category": "huggingface_handler",
+                            "type": "check_size",
+                            "status": "failed",
+                        }
+                    }
                 )
+                logger.error(f"Dataset size check failed: {e}", extra=check_size_error)
                 raise HTTPException(
-                    status_code=400, detail=f"Dataset size check failed: {size_error}"
+                    status_code=400, detail=f"Dataset size check failed: {e}"
                 )
 
-            # Step 2: Download
-            api = HfApi()
-            info = api.repo_info(dataset_name, repo_type="dataset", revision=revision)
-
             temp_dir = storage_handler.temp_dir
-            logger.info(f"Using PVC-backed temp directory: {temp_dir}")
 
             local_dataset_dir = temp_dir
             local_dataset_dir.mkdir(parents=True, exist_ok=True)
-            logger.info(
-                f"Saving dataset splits to '{local_dataset_dir}' in Parquet format..."
-            )
-
-            logger.info(
-                f"Downloading raw files for dataset '{dataset_name}' at revision '{revision}'..."
-            )
 
             # snapshot_download(
             #     repo_id=dataset_name,
@@ -322,66 +310,153 @@ class HuggingFaceHandler:
                         dataset_name, dataset_config, cache_dir=str(temp_dir / "cache")
                     )
                 except Exception as e:
+                    download_error_extra = base_extra.copy()
+                    download_error_extra.update(
+                        {
+                            "event": {
+                                "category": "huggingface_handler",
+                                "type": "download_error",
+                                "status": "failed",
+                            }
+                        }
+                    )
                     logger.error(
-                        f"Error loading dataset '{dataset_name}': {e}", exc_info=True
+                        f"Error loading dataset:{e}",
+                        exc_info=True,
+                        extra=download_error_extra,
                     )
                     raise HTTPException(
                         status_code=500,
                         detail=f"Error loading dataset '{dataset_name}': {e}",
                     )
-                renamables = self._check_schema(ds["train"].features, dataset_tag)
 
-                if renamables:
-                    ds = self._rename_columns(ds, renamables)
-                    if dataset_tag == "text_generation":
-                        self._process_text_generation(ds, local_dataset_dir)
-                    elif dataset_tag == "image_classification":
-                        logger.info("Processing dataset for Image Classification task.")
-                        # This list will collect metadata for all splits before final saving.
-                        all_metadata = []
+                downloaded_dataset_extra = base_extra.copy()
 
-                        updated_ds = self.save_images(ds, local_dataset_dir)
-
-                        # --- Step 4: Combine and save the final metadata ---
-                        all_splits_metadata = []
-                        for split_name, split_dataset in updated_ds.items():
-                            # Convert to pandas to easily add the 'split' column
-                            split_df = split_dataset.to_pandas()
-                            split_df["split"] = split_name
-                            all_splits_metadata.append(split_df)
-
-                        # Concatenate all dataframes from all splits
-                        final_metadata_df = pd.concat(
-                            all_splits_metadata, ignore_index=True
-                        )
-
-                        if final_metadata_df.empty:
-                            logger.warning(
-                                "No valid images were processed. Metadata file will not be created."
-                            )
-                        else:
-                            logger.info(
-                                f"Saving metadata for {len(final_metadata_df)} images to Parquet file."
-                            )
-                            final_metadata_df.to_parquet(
-                                local_dataset_dir / "metadata.parquet", index=False
-                            )
-                        logger.info(
-                            "Image classification dataset restructuring complete."
-                        )
-                    # raise HTTPException(
-                    #     status_code=400,
-                    #     detail=f"The dataset '{dataset_name}' with config '{dataset_config}' does not contain any is not inferable data.",
-                    # )
-                else:
-                    logger.info(
-                        f"Skipping schema enforcement for dataset '{dataset_name}'."
-                    )
-            else:
-                logger.info(
-                    f"Dataset '{dataset_name}' does not have inferable features; skipping schema check."
+                downloaded_dataset_extra.update(
+                    {
+                        "event": {
+                            "category": "huggingface_handler",
+                            "type": "dataset_downloaded",
+                            "status": "success",
+                        }
+                    }
                 )
 
+                logger.info(
+                    "huggingface dataset downloaded successfully",
+                    extra=downloaded_dataset_extra,
+                )
+
+                if restructure==True:
+                    renamables = self._check_schema(ds["train"].features, dataset_tag)
+
+                    if renamables:
+                        ds = self._rename_columns(ds, renamables)
+                        if dataset_tag == "text_generation":
+                            try:
+                                self._process_text_generation(ds, local_dataset_dir)
+
+                                text_generation_success = base_extra.copy()
+                                text_generation_success.update(
+                                    {
+                                        "event": {
+                                            "category": "huggingface_handler",
+                                            "type": "restructure",
+                                            "status": "success",
+                                        }
+                                    }
+                                )
+                                logger.info(
+                                    "text data restructured successfully",
+                                    extra=text_generation_success,
+                                )
+                            except Exception as e:
+                                text_generation_error = base_extra.copy()
+                                text_generation_error.update(
+                                    {
+                                        "event": {
+                                            "category": "huggingface_handler",
+                                            "type": "restructure",
+                                            "status": "success",
+                                        }
+                                    }
+                                )
+                                logger.error(
+                                    f"text restructure failed:{e}",
+                                    extra=text_generation_error,
+                                )
+
+                        elif dataset_tag == "image_classification":
+                            try:
+                                updated_ds = self.save_images(ds, local_dataset_dir)
+
+                                # --- Step 4: Combine and save the final metadata ---
+                                all_splits_metadata = []
+                                for split_name, split_dataset in updated_ds.items():
+                                    # Convert to pandas to easily add the 'split' column
+                                    split_df = split_dataset.to_pandas()
+                                    split_df["split"] = split_name
+                                    all_splits_metadata.append(split_df)
+
+                                # Concatenate all dataframes from all splits
+                                final_metadata_df = pd.concat(
+                                    all_splits_metadata, ignore_index=True
+                                )
+
+                                if not final_metadata_df.empty:
+                                    logger.info(
+                                        f"Saving metadata for {len(final_metadata_df)} images to Parquet file."
+                                    )
+                                    final_metadata_df.to_parquet(
+                                        local_dataset_dir / "metadata.parquet", index=False
+                                    )
+                                else:
+                                    raise ValueError("metadata dataframe is empty")
+
+                                image_resturcture_success = base_extra.copy()
+                                image_resturcture_success.update(
+                                    {
+                                        "event": {
+                                            "category": "huggingface_handler",
+                                            "type": "resturcture",
+                                            "status": "success",
+                                        }
+                                    }
+                                )
+
+                            except Exception as e:
+                                image_resturcture_error = base_extra.copy()
+                                image_resturcture_error.update(
+                                    {
+                                        "event": {
+                                            "category": "huggingface_handler",
+                                            "type": "restructure_failed",
+                                            "status": "success",
+                                        }
+                                    }
+                                )
+                                logger.error(
+                                    f"error in restructuring huggingface dataset: {e}",
+                                    extra=image_resturcture_error,
+                                )
+                    else:
+                        check_schema_error = base_extra.copy()
+                        check_schema_error.update(
+                            {
+                                "event": {
+                                    "category": "huggingface_handler",
+                                    "type": "column_rename",
+                                    "status": "failed",
+                                }
+                            }
+                        )
+                        logger.error(
+                            "schema enforcement failed for huggingface dataset.",
+                            extra=check_schema_error,
+                        )
+                else:
+                    logger.info("restructure=False: not doing restructuring")
+
             api = HfApi()
             info = api.repo_info(dataset_name, repo_type="dataset")
 
@@ -404,13 +479,44 @@ class HuggingFaceHandler:
                 dataset_tag=dataset_tag,
             )
 
-            # Store valid files
-            stored_path = storage_handler.store_dataset(
-                local_dataset_dir,
-                metadata,
-                s3_config=s3_config,
-                clearml_config=clearml_config,
+            base_extra["dataset"]["id"] = ds_id
+
+            metadata_extra = base_extra.copy()
+            metadata_extra.update(
+                {
+                    "event": {
+                        "category": "huggingface_handler",
+                        "type": "metadata",
+                        "status": "success",
+                    }
+                }
             )
+            logger.info("metadata object created", extra=metadata_extra)
+
+            # Store valid files
+
+            try:
+                stored_path = storage_handler.store_dataset(
+                    local_dataset_dir,
+                    metadata,
+                    s3_config=s3_config,
+                    clearml_config=clearml_config,
+                )
+            except Exception as e:
+                error_saving_extra = base_extra.copy()
+                error_saving_extra.update(
+                    {
+                        "event": {
+                            "category": "huggingface_handler",
+                            "type": "error_saving",
+                            "status": "failed",
+                        }
+                    }
+                )
+                logger.error(
+                    f"error in saving huggingface dataset: {e}",
+                    extra=error_saving_extra,
+                )
 
             #  Construct final response
             response = {
@@ -419,8 +525,6 @@ class HuggingFaceHandler:
                 "dataset_id": ds_id,
                 "stored_path": stored_path,
             }
-            # if error_details:
-            #     response["validation_errors"] = error_details
 
             return response
 
