@@ -23,7 +23,7 @@ logging.basicConfig(
     level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
 )
 
-CLEARML_API_URL = "http://144.172.105.98:30001"
+CLEARML_API_URL = "http://144.172.105.98:30003"
 USER_MANAGEMENT_API = "http://144.172.105.98:30009"
 
 def get_user_info(username, user_token):
@@ -372,7 +372,7 @@ def s3_download(
     clearml_access_key = user_metadata["clearml_access_key"]
     clearml_secret_key = user_metadata["clearml_secret_key"]
     # s3_endpoint_url = user_metadata["s3_endpoint_url"]
-    s3_endpoint_url =os.environ.get("CEPH_ENDPOINT_URL")
+    s3_endpoint_url ="http://144.172.105.98:7000"
 
 
 
